package app.fedilab.android.drawers;
/* Copyright 2017 Thomas Schneider
 *
 * This file is a part of Fedilab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * Fedilab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with Fedilab; if not,
 * see <http://www.gnu.org/licenses>. */


import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.bumptech.glide.Glide;

import java.lang.ref.WeakReference;
import java.util.List;

import app.fedilab.android.R;
import app.fedilab.android.activities.TootActivity;
import app.fedilab.android.client.Entities.Account;


/**
 * Created by Thomas on 25/10/2017.
 * Adapter for accounts when replying
 */
public class AccountsReplyAdapter extends BaseAdapter {

    private List<Account> accounts;
    private boolean[] checked;
    private WeakReference<Activity> activityWeakReference;

    public AccountsReplyAdapter(WeakReference<Activity> activityWeakReference, List<Account> accounts, boolean[] checked) {
        this.accounts = accounts;
        this.checked = checked;
        this.activityWeakReference = activityWeakReference;
    }

    public AccountsReplyAdapter(WeakReference<Activity> activityWeakReference, List<Account> accounts, List<Boolean> checked) {
        this.accounts = accounts;
        this.checked = new boolean[checked.size()];
        int index = 0;
        for (Boolean val : checked) {
            this.checked[index++] = val;
        }
        this.activityWeakReference = activityWeakReference;
    }

    @Override
    public int getCount() {
        return accounts.size();
    }

    @Override
    public Account getItem(int position) {
        return accounts.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }


    @NonNull
    @Override
    public View getView(final int position, View convertView, @NonNull ViewGroup parent) {
        LayoutInflater layoutInflater = LayoutInflater.from(activityWeakReference.get());
        final Account account = accounts.get(position);
        final ViewHolder holder;
        if (convertView == null) {
            convertView = layoutInflater.inflate(R.layout.drawer_account_reply, parent, false);
            holder = new ViewHolder();
            holder.account_pp = convertView.findViewById(R.id.account_pp);
            holder.account_dn = convertView.findViewById(R.id.account_dn);
            holder.checkbox = convertView.findViewById(R.id.checkbox);

            holder.account_container = convertView.findViewById(R.id.account_container);
            convertView.setTag(holder);
        } else {
            holder = (ViewHolder) convertView.getTag();
        }
        holder.checkbox.setOnCheckedChangeListener((compoundButton, isChecked) -> {
            try {
                ((TootActivity) activityWeakReference.get()).changeAccountReply(isChecked, "@" + account.getAcct());
                checked[position] = isChecked;
            } catch (Exception ignored) {
            }
        });
        holder.checkbox.setChecked(checked[position]);
        holder.account_dn.setText(String.format("@%s", account.getAcct()));
        holder.account_container.setOnClickListener(view -> holder.checkbox.performClick());
        //Profile picture
        Glide.with(holder.account_pp.getContext())
                .load(account.getAvatar())
                .into(holder.account_pp);
        return convertView;
    }


    private static class ViewHolder {
        ImageView account_pp;
        TextView account_dn;
        CheckBox checkbox;
        LinearLayout account_container;
    }


}